/*==========================================================================*/
/*              Reinier Bakels                                              */
/*              IBM Netherlands                                             */
/*              EAMSVM1(NL04191)                                            */
/*                                                                          */
/*              (C) Copyright IBM Corporation 1998                          */
/*==========================================================================*/
/* Interprets MQI return codes                                              */
/*==========================================================================*/
/* v1.0  17 Jan 1998  Created                                               */
/*==========================================================================*/
/* Module Name: MQCHECK.C                                                   */
/*==========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <cmqc.h>

typedef struct {
   short  code ;
   char  *text ;
} CODETABLE, *PCODETABLE;

#ifdef __cplusplus
   extern "C"
#endif
int compareCode (const void * element1, const void * element2) { return *(const short*)element1-*(const short*)element2; }

unsigned long mqCheck(char * text , unsigned long compCode , unsigned long reasonCode )
{
   CODETABLE codeTable [] = {
   /*    0L */   MQRC_NONE                      , "none"
   /* 2001L */ , MQRC_ALIAS_BASE_Q_TYPE_ERROR   , "alias base Q type error"
   /* 2002L */ , MQRC_ALREADY_CONNECTED         , "already connected"
   /* 2003L */ , MQRC_BACKED_OUT                , "backed out"
   /* 2004L */ , MQRC_BUFFER_ERROR              , "buffer error"
   /* 2005L */ , MQRC_BUFFER_LENGTH_ERROR       , "buffer length error"
   /* 2006L */ , MQRC_CHAR_ATTR_LENGTH_ERROR    , "char attr length error"
   /* 2007L */ , MQRC_CHAR_ATTRS_ERROR          , "char attrs error"
   /* 2008L */ , MQRC_CHAR_ATTRS_TOO_SHORT      , "char attrs too short"
   /* 2009L */ , MQRC_CONNECTION_BROKEN         , "connection broken"
   /* 2010L */ , MQRC_DATA_LENGTH_ERROR         , "data length error"
   /* 2011L */ , MQRC_DYNAMIC_Q_NAME_ERROR      , "dynamic Q name error"
   /* 2012L */ , MQRC_ENVIRONMENT_ERROR         , "environment error"
   /* 2013L */ , MQRC_EXPIRY_ERROR              , "expiry error"
   /* 2014L */ , MQRC_FEEDBACK_ERROR            , "feedback error"
   /* 2016L */ , MQRC_GET_INHIBITED             , "get inhibited"
   /* 2017L */ , MQRC_HANDLE_NOT_AVAILABLE      , "handle not available"
   /* 2018L */ , MQRC_HCONN_ERROR               , "hconn error"
   /* 2019L */ , MQRC_HOBJ_ERROR                , "hobj error"
   /* 2020L */ , MQRC_INHIBIT_VALUE_ERROR       , "inhibit value error"
   /* 2021L */ , MQRC_INT_ATTR_COUNT_ERROR      , "int attr count error"
   /* 2022L */ , MQRC_INT_ATTR_COUNT_TOO_SMALL  , "int attr count too small"
   /* 2023L */ , MQRC_INT_ATTRS_ARRAY_ERROR     , "int attrs array error"
   /* 2024L */ , MQRC_SYNCPOINT_LIMIT_REACHED   , "syncpoint limit reached"
   /* 2025L */ , MQRC_MAX_CONNS_LIMIT_REACHED   , "max conns limit reached"
   /* 2026L */ , MQRC_MD_ERROR                  , "md error"
   /* 2027L */ , MQRC_MISSING_REPLY_TO_Q        , "missing reply to Q"
   /* 2029L */ , MQRC_MSG_TYPE_ERROR            , "msg type error"
   /* 2030L */ , MQRC_MSG_TOO_BIG_FOR_Q         , "msg too big for Q"
   /* 2031L */ , MQRC_MSG_TOO_BIG_FOR_Q_MGR     , "msg too big for Q mgr"
   /* 2033L */ , MQRC_NO_MSG_AVAILABLE          , "no msg available"
   /* 2034L */ , MQRC_NO_MSG_UNDER_CURSOR       , "no msg under cursor"
   /* 2035L */ , MQRC_NOT_AUTHORIZED            , "not authorized"
   /* 2036L */ , MQRC_NOT_OPEN_FOR_BROWSE       , "not open for browse"
   /* 2037L */ , MQRC_NOT_OPEN_FOR_INPUT        , "not open for input"
   /* 2038L */ , MQRC_NOT_OPEN_FOR_INQUIRE      , "not open for inquire"
   /* 2039L */ , MQRC_NOT_OPEN_FOR_OUTPUT       , "not open for output"
   /* 2040L */ , MQRC_NOT_OPEN_FOR_SET          , "not open for set"
   /* 2041L */ , MQRC_OBJECT_CHANGED            , "object changed"
   /* 2042L */ , MQRC_OBJECT_IN_USE             , "object in use"
   /* 2043L */ , MQRC_OBJECT_TYPE_ERROR         , "object type error"
   /* 2044L */ , MQRC_OD_ERROR                  , "OD error"
   /* 2045L */ , MQRC_OPTION_NOT_VALID_FOR_TYPE , "option not valid for type"
   /* 2046L */ , MQRC_OPTIONS_ERROR             , "options error"
   /* 2047L */ , MQRC_PERSISTENCE_ERROR         , "persistence error"
   /* 2048L */ , MQRC_PERSISTENT_NOT_ALLOWED    , "persistent not allowed"
   /* 2049L */ , MQRC_PRIORITY_EXCEEDS_MAXIMUM  , "priority exceeds maximum"
   /* 2050L */ , MQRC_PRIORITY_ERROR            , "priority error"
   /* 2051L */ , MQRC_PUT_INHIBITED             , "put inhibited"
   /* 2052L */ , MQRC_Q_DELETED                 , "Q deleted"
   /* 2053L */ , MQRC_Q_FULL                    , "Q full"
   /* 2055L */ , MQRC_Q_NOT_EMPTY               , "Q not empty"
   /* 2056L */ , MQRC_Q_SPACE_NOT_AVAILABLE     , "Q space not available"
   /* 2057L */ , MQRC_Q_TYPE_ERROR              , "Q type error"
   /* 2058L */ , MQRC_Q_MGR_NAME_ERROR          , "Q mgr name error"
   /* 2059L */ , MQRC_Q_MGR_NOT_AVAILABLE       , "Q mgr not available"
   /* 2061L */ , MQRC_REPORT_OPTIONS_ERROR      , "report options error"
   /* 2062L */ , MQRC_SECOND_MARK_NOT_ALLOWED   , "second mark not allowed"
   /* 2063L */ , MQRC_SECURITY_ERROR            , "security error"
   /* 2065L */ , MQRC_SELECTOR_COUNT_ERROR      , "selector count error"
   /* 2066L */ , MQRC_SELECTOR_LIMIT_EXCEEDED   , "selector limit exceeded"
   /* 2067L */ , MQRC_SELECTOR_ERROR            , "selector error"
   /* 2068L */ , MQRC_SELECTOR_NOT_FOR_TYPE     , "selector not for type"
   /* 2069L */ , MQRC_SIGNAL_OUTSTANDING        , "signal outstanding"
   /* 2070L */ , MQRC_SIGNAL_REQUEST_ACCEPTED   , "signal request accepted"
   /* 2071L */ , MQRC_STORAGE_NOT_AVAILABLE     , "storage not available"
   /* 2072L */ , MQRC_SYNCPOINT_NOT_AVAILABLE   , "syncpoint not available"
   /* 2075L */ , MQRC_TRIGGER_CONTROL_ERROR     , "trigger control error"
   /* 2076L */ , MQRC_TRIGGER_DEPTH_ERROR       , "trigger depth error"
   /* 2077L */ , MQRC_TRIGGER_MSG_PRIORITY_ERR  , "trigger msg priority err"
   /* 2078L */ , MQRC_TRIGGER_TYPE_ERROR        , "trigger type error"
   /* 2079L */ , MQRC_TRUNCATED_MSG_ACCEPTED    , "truncated msg accepted"
   /* 2080L */ , MQRC_TRUNCATED_MSG_FAILED      , "truncated msg failed"
   /* 2082L */ , MQRC_UNKNOWN_ALIAS_BASE_Q      , "unknown alias base Q"
   /* 2085L */ , MQRC_UNKNOWN_OBJECT_NAME       , "unknown object name"
   /* 2086L */ , MQRC_UNKNOWN_OBJECT_Q_MGR      , "unknown object Q mgr"
   /* 2087L */ , MQRC_UNKNOWN_REMOTE_Q_MGR      , "unknown remote Q mgr"
   /* 2090L */ , MQRC_WAIT_INTERVAL_ERROR       , "wait interval error"
   /* 2091L */ , MQRC_XMIT_Q_TYPE_ERROR         , "xmit Q type error"
   /* 2092L */ , MQRC_XMIT_Q_USAGE_ERROR        , "xmit Q usage error"
   /* 2093L */ , MQRC_NOT_OPEN_FOR_PASS_ALL     , "not open for pass all"
   /* 2094L */ , MQRC_NOT_OPEN_FOR_PASS_IDENT   , "not open for pass ident"
   /* 2095L */ , MQRC_NOT_OPEN_FOR_SET_ALL      , "not open for set all"
   /* 2096L */ , MQRC_NOT_OPEN_FOR_SET_IDENT    , "not open for set ident"
   /* 2097L */ , MQRC_CONTEXT_HANDLE_ERROR      , "context handle error"
   /* 2098L */ , MQRC_CONTEXT_NOT_AVAILABLE     , "context not available"
   /* 2099L */ , MQRC_SIGNAL1_ERROR             , "signal1 error"
   /* 2100L */ , MQRC_OBJECT_ALREADY_EXISTS     , "object already exists"
   /* 2101L */ , MQRC_OBJECT_DAMAGED            , "object damaged"
   /* 2102L */ , MQRC_RESOURCE_PROBLEM          , "resource problem"
   /* 2103L */ , MQRC_ANOTHER_Q_MGR_CONNECTED   , "another Q mgr connected"
   /* 2104L */ , MQRC_UNKNOWN_REPORT_OPTION     , "unknown report option"
   /* 2105L */ , MQRC_STORAGE_CLASS_ERROR       , "storage class error"
   /* 2106L */ , MQRC_COD_NOT_VALID_FOR_XCF_Q   , "cod not valid for xcf Q"
   /* 2107L */ , MQRC_XWAIT_CANCELED            , "xwait canceled"
   /* 2108L */ , MQRC_XWAIT_ERROR               , "xwait error"
   /* 2109L */ , MQRC_SUPPRESSED_BY_EXIT        , "suppressed by exit"
   /* 2110L */ , MQRC_FORMAT_ERROR              , "format error"
   /* 2111L */ , MQRC_SOURCE_CCSID_ERROR        , "source ccsid error"
   /* 2112L */ , MQRC_SOURCE_INTEGER_ENC_ERROR  , "source integer enc error"
   /* 2113L */ , MQRC_SOURCE_DECIMAL_ENC_ERROR  , "source decimal enc error"
   /* 2114L */ , MQRC_SOURCE_FLOAT_ENC_ERROR    , "source float enc error"
   /* 2115L */ , MQRC_TARGET_CCSID_ERROR        , "target ccsid error"
   /* 2116L */ , MQRC_TARGET_INTEGER_ENC_ERROR  , "target integer enc error"
   /* 2117L */ , MQRC_TARGET_DECIMAL_ENC_ERROR  , "target decimal enc error"
   /* 2118L */ , MQRC_TARGET_FLOAT_ENC_ERROR    , "target float enc error"
   /* 2119L */ , MQRC_NOT_CONVERTED             , "not converted"
   /* 2120L */ , MQRC_CONVERTED_MSG_TOO_BIG     , "converted msg too big, truncated"
// /* 2120L */ , MQRC_CONVERTED_MSG_TOO_BIG     , "converted msg too big"
// /* 2120L */ , MQRC_TRUNCATED                 , "truncated"
   /* 2121L */ , MQRC_NO_EXTERNAL_PARTICIPANTS  , "no external participants"
   /* 2122L */ , MQRC_PARTICIPANT_NOT_AVAILABLE , "participant not available"
   /* 2123L */ , MQRC_OUTCOME_MIXED             , "outcome mixed"
   /* 2124L */ , MQRC_OUTCOME_PENDING           , "outcome pending"
   /* 2125L */ , MQRC_BRIDGE_STARTED            , "bridge started"
   /* 2126L */ , MQRC_BRIDGE_STOPPED            , "bridge stopped"
   /* 2127L */ , MQRC_ADAPTER_STORAGE_SHORTAGE  , "adapter storage shortage"
   /* 2128L */ , MQRC_UOW_IN_PROGRESS           , "uow in progress"
   /* 2129L */ , MQRC_ADAPTER_CONN_LOAD_ERROR   , "adapter conn load error"
   /* 2130L */ , MQRC_ADAPTER_SERV_LOAD_ERROR   , "adapter serv load error"
   /* 2131L */ , MQRC_ADAPTER_DEFS_ERROR        , "adapter defs error"
   /* 2132L */ , MQRC_ADAPTER_DEFS_LOAD_ERROR   , "adapter defs load error"
   /* 2133L */ , MQRC_ADAPTER_CONV_LOAD_ERROR   , "adapter conv load error"
   /* 2134L */ , MQRC_BO_ERROR                  , "BO error"
   /* 2135L */ , MQRC_DH_ERROR                  , "DH error"
   /* 2136L */ , MQRC_MULTIPLE_REASONS          , "multiple reasons"
   /* 2137L */ , MQRC_OPEN_FAILED               , "open failed"
   /* 2138L */ , MQRC_ADAPTER_DISC_LOAD_ERROR   , "adapter disc load error"
   /* 2139L */ , MQRC_CNO_ERROR                 , "CNO error"
   /* 2140L */ , MQRC_CICS_WAIT_FAILED          , "CICS wait failed"
   /* 2141L */ , MQRC_DLH_ERROR                 , "dlh error"
   /* 2142L */ , MQRC_HEADER_ERROR              , "header error"
   /* 2143L */ , MQRC_SOURCE_LENGTH_ERROR       , "source length error"
   /* 2144L */ , MQRC_TARGET_LENGTH_ERROR       , "target length error"
   /* 2145L */ , MQRC_SOURCE_BUFFER_ERROR       , "source buffer error"
   /* 2146L */ , MQRC_TARGET_BUFFER_ERROR       , "target buffer error"
   /* 2148L */ , MQRC_IIH_ERROR                 , "IIH error"
   /* 2149L */ , MQRC_PCF_ERROR                 , "PCF error"
   /* 2150L */ , MQRC_DBCS_ERROR                , "DBCS error"
   /* 2152L */ , MQRC_OBJECT_NAME_ERROR         , "object name error"
   /* 2153L */ , MQRC_OBJECT_Q_MGR_NAME_ERROR   , "object Q mgr name error"
   /* 2154L */ , MQRC_RECS_PRESENT_ERROR        , "recs present error"
   /* 2155L */ , MQRC_OBJECT_RECORDS_ERROR      , "object records error"
   /* 2156L */ , MQRC_RESPONSE_RECORDS_ERROR    , "response records error"
   /* 2157L */ , MQRC_ASID_MISMATCH             , "asid mismatch"
   /* 2158L */ , MQRC_PMO_RECORD_FLAGS_ERROR    , "pmo record flags error"
   /* 2159L */ , MQRC_PUT_MSG_RECORDS_ERROR     , "put msg records error"
   /* 2160L */ , MQRC_CONN_ID_IN_USE            , "conn id in use"
   /* 2161L */ , MQRC_Q_MGR_QUIESCING           , "Q mgr quiescing"
   /* 2162L */ , MQRC_Q_MGR_STOPPING            , "Q mgr stopping"
   /* 2163L */ , MQRC_DUPLICATE_RECOV_COORD     , "duplicate recov coord"
   /* 2173L */ , MQRC_PMO_ERROR                 , "pmo error"
   /* 2182L */ , MQRC_API_EXIT_NOT_FOUND        , "API exit not found"
   /* 2183L */ , MQRC_API_EXIT_LOAD_ERROR       , "API exit load error"
   /* 2184L */ , MQRC_REMOTE_Q_NAME_ERROR       , "remote Q name error"
   /* 2185L */ , MQRC_INCONSISTENT_PERSISTENCE  , "inconsistent persistence"
   /* 2186L */ , MQRC_GMO_ERROR                 , "GMO error"
   /* 2191L */ , MQRC_TMC_ERROR                 , "TMC error"
   /* 2192L */ , MQRC_PAGESET_FULL              , "pageset full"
   /* 2193L */ , MQRC_PAGESET_ERROR             , "pageset error"
   /* 2194L */ , MQRC_NAME_NOT_VALID_FOR_TYPE   , "name not valid for type"
   /* 2195L */ , MQRC_UNEXPECTED_ERROR          , "unexpected error"
   /* 2196L */ , MQRC_UNKNOWN_XMIT_Q            , "unknown xmit Q"
   /* 2197L */ , MQRC_UNKNOWN_DEF_XMIT_Q        , "unknown def xmit Q"
   /* 2198L */ , MQRC_DEF_XMIT_Q_TYPE_ERROR     , "def xmit Q type error"
   /* 2199L */ , MQRC_DEF_XMIT_Q_USAGE_ERROR    , "def xmit Q usage error"
   /* 2201L */ , MQRC_NAME_IN_USE               , "name in use"
   /* 2202L */ , MQRC_CONNECTION_QUIESCING      , "connection quiescing"
   /* 2203L */ , MQRC_CONNECTION_STOPPING       , "connection stopping"
   /* 2204L */ , MQRC_ADAPTER_NOT_AVAILABLE     , "adapter not available"
   /* 2206L */ , MQRC_MSG_ID_ERROR              , "msg id error"
   /* 2207L */ , MQRC_CORREL_ID_ERROR           , "correl id error"
   /* 2208L */ , MQRC_FILE_SYSTEM_ERROR         , "file system error"
   /* 2209L */ , MQRC_NO_MSG_LOCKED             , "no msg locked"
   /* 2216L */ , MQRC_FILE_NOT_AUDITED          , "file not audited"
   /* 2217L */ , MQRC_CONNECTION_NOT_AUTHORIZED , "connection not authorized"
   /* 2218L */ , MQRC_MSG_TOO_BIG_FOR_CHANNEL   , "msg too big for channel"
   /* 2219L */ , MQRC_CALL_IN_PROGRESS          , "call in progress"
   /* 2220L */ , MQRC_RMH_ERROR                 , "RMH error"
   /* 2222L */ , MQRC_Q_MGR_ACTIVE              , "Q mgr active"
   /* 2223L */ , MQRC_Q_MGR_NOT_ACTIVE          , "Q mgr not active"
   /* 2224L */ , MQRC_Q_DEPTH_HIGH              , "Q depth high"
   /* 2225L */ , MQRC_Q_DEPTH_LOW               , "Q depth low"
   /* 2226L */ , MQRC_Q_SERVICE_INTERVAL_HIGH   , "Q service interval high"
   /* 2227L */ , MQRC_Q_SERVICE_INTERVAL_OK     , "Q service interval OK"
   /* 2233L */ , MQRC_CHANNEL_AUTO_DEF_OK       , "channel auto def OK"
   /* 2234L */ , MQRC_CHANNEL_AUTO_DEF_ERROR    , "channel auto def error"
   /* 2235L */ , MQRC_CFH_ERROR                 , "CFH error"
   /* 2236L */ , MQRC_CFIL_ERROR                , "CFIL error"
   /* 2237L */ , MQRC_CFIN_ERROR                , "CFIN error"
   /* 2238L */ , MQRC_CFSL_ERROR                , "CFSL error"
   /* 2239L */ , MQRC_CFST_ERROR                , "CFST error"
   /* 2241L */ , MQRC_INCOMPLETE_GROUP          , "incomplete group"
   /* 2242L */ , MQRC_INCOMPLETE_MSG            , "incomplete msg"
   /* 2243L */ , MQRC_INCONSISTENT_CCSIDS       , "inconsistent ccsids"
   /* 2244L */ , MQRC_INCONSISTENT_ENCODINGS    , "inconsistent encodings"
   /* 2245L */ , MQRC_INCONSISTENT_UOW          , "inconsistent uow"
   /* 2246L */ , MQRC_INVALID_MSG_UNDER_CURSOR  , "invalid msg under cursor"
   /* 2247L */ , MQRC_MATCH_OPTIONS_ERROR       , "match options error"
   /* 2248L */ , MQRC_MDE_ERROR                 , "mde error"
   /* 2249L */ , MQRC_MSG_FLAGS_ERROR           , "msg flags error"
   /* 2250L */ , MQRC_MSG_SEQ_NUMBER_ERROR      , "msg seq number error"
   /* 2251L */ , MQRC_OFFSET_ERROR              , "offset error"
   /* 2252L */ , MQRC_ORIGINAL_LENGTH_ERROR     , "original length error"
   /* 2253L */ , MQRC_SEGMENT_LENGTH_ZERO       , "segment length zero"
   /* 2255L */ , MQRC_UOW_NOT_AVAILABLE         , "uow not available"
   /* 2256L */ , MQRC_WRONG_GMO_VERSION         , "wrong gmo version"
   /* 2257L */ , MQRC_WRONG_MD_VERSION          , "wrong md version"
   /* 2258L */ , MQRC_GROUP_ID_ERROR            , "group id error"
   /* 2259L */ , MQRC_INCONSISTENT_BROWSE       , "inconsistent browse"
   /* 2260L */ , MQRC_XQH_ERROR                 , "xqh error"
   /* 2261L */ , MQRC_SRC_ENV_ERROR             , "src env error"
   /* 2262L */ , MQRC_SRC_NAME_ERROR            , "src name error"
   /* 2263L */ , MQRC_DEST_ENV_ERROR            , "dest env error"
   /* 2264L */ , MQRC_DEST_NAME_ERROR           , "dest name error"
   /* 2265L */ , MQRC_TM_ERROR                  , "tm error"
   /* 2280L */ , MQRC_HCONFIG_ERROR             , "hconfig error"
   /* 2281L */ , MQRC_FUNCTION_ERROR            , "function error"
   /* 2282L */ , MQRC_CHANNEL_STARTED           , "channel started"
   /* 2283L */ , MQRC_CHANNEL_STOPPED           , "channel stopped"
   /* 2284L */ , MQRC_CHANNEL_CONV_ERROR        , "channel conv error"
   /* 2285L */ , MQRC_SERVICE_NOT_AVAILABLE     , "service not available"
   /* 2286L */ , MQRC_INITIALIZATION_FAILED     , "initialization failed"
   /* 2287L */ , MQRC_TERMINATION_FAILED        , "termination failed"
   /* 2288L */ , MQRC_UNKNOWN_Q_NAME            , "unknown Q name"
   /* 2289L */ , MQRC_SERVICE_ERROR             , "service error"
   /* 2290L */ , MQRC_Q_ALREADY_EXISTS          , "Q already exists"
   /* 2291L */ , MQRC_USER_ID_NOT_AVAILABLE     , "user id not available"
   /* 2292L */ , MQRC_UNKNOWN_ENTITY            , "unknown entity"
   /* 2293L */ , MQRC_UNKNOWN_AUTH_ENTITY       , "unknown auth entity"
   /* 2294L */ , MQRC_UNKNOWN_REF_OBJECT        , "unknown ref object"
   /* 2295L */ , MQRC_CHANNEL_ACTIVATED         , "channel activated"
   /* 2296L */ , MQRC_CHANNEL_NOT_ACTIVATED     , "channel not activated"
   };
   char * completions[] = {
        "ended with unknown status"
      , "OK"
      , "ended with warning"
      , "failed"
   } ;
   PCODETABLE codeTableEntry ;
   char * reason     = 0 ;
   char   buffer[33] ={0};
   short  shortCode  = reasonCode ;

   // MQCC_OK      0L
   // MQCC_WARNING 1L
   // MQCC_FAILED  2L
   // MQCC_UNKNOWN (-1L)

   if (text) {
      printf("%-8s ",text);
   } else {
      printf("MQ call ");
   } /* endif */

   if (compCode) {
      if ( codeTableEntry = (PCODETABLE)bsearch(&shortCode,codeTable,sizeof(codeTable)/sizeof(CODETABLE),sizeof(CODETABLE),compareCode) ) {
         reason = codeTableEntry->text ;
      } else {
         sprintf( reason = buffer ,"(unknown code %i)",reasonCode);
      } /* endif */
      printf("%s, reason: %s.\n"
       , ++compCode< sizeof(completions)/sizeof(*completions)? completions[compCode] : "?"
       , reason
      );
   } else {
      printf("successful.\n");
   } /* endif */

   return compCode ;
}
